Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;

ENTITY Test_SCCC IS
    END Test_SCCC;

ARCHITECTURE behavior OF Test_SCCC IS

    constant Tpw_clk : time := 10 ns;

    -- Component Declaration of the Module Under Test (MUT)
    component sequencer_timer_v2 is 
        port( 
                clk                 : in std_logic;
                nreset              : in std_logic;
                Inhibit_n           : in std_logic;
                Cap_Volt_Inh        : in std_logic; 
                PULSE_WIDTH         : in std_logic_vector(7 downto 0);
                PULSE_SEPARATION    : in std_logic_vector(7 downto 0);

                Seq_Out_1n          : out std_logic;
                Seq_Out_2n          : out std_logic;
                Seq_Out_3n          : out std_logic;
                Seq_Out_4n          : out std_logic;
                Seq_Out_5n          : out std_logic;
                next_out            : out std_logic;
                start_out           : out std_logic;
                enable_out          : out std_logic;
                stop_out            : out std_logic;
                State_out           : out std_logic_vector(3 downto 0)
            );
    end component;

    -- inputs
    constant Pulse_Width    : std_logic_vector(7 downto 0) := X"16"; 
    constant Pulse_Sep      : std_logic_vector(7 downto 0) := X"08";

    signal clk              : std_logic := '0';
    signal nreset           : std_logic := '1';
    signal inhibit_n        : std_logic := '1';
    signal vcap_inhib       : std_logic := '0';

    -- outputs
    signal seq_1            : std_logic := '0';
    signal seq_2            : std_logic := '0';
    signal seq_3            : std_logic := '0';
    signal seq_4            : std_logic := '0';
    signal seq_5            : std_logic := '0';
    signal nnext            : std_logic := '0';
    signal start            : std_logic := '0';
    signal enable           : std_logic := '0';
    signal sstop            : std_logic := '0';
    signal state            : std_logic_vector(3 downto 0) := X"0";

BEGIN
    -- Instantiate the Module Under Test
    MUT: sequencer_timer_v2
    port map ( 	
                 clk				=> clk,
                 nreset				=> nreset,
                 Inhibit_n		    => inhibit_n,
                 Cap_Volt_Inh		=> vcap_inhib,
                 PULSE_WIDTH		=> Pulse_Width,
                 PULSE_SEPARATION	=> Pulse_Sep,

                 Seq_Out_1n			=> seq_1,
                 Seq_Out_2n			=> seq_2,
                 Seq_Out_3n			=> seq_3,
                 Seq_Out_4n			=> seq_4,
                 Seq_Out_5n			=> seq_5,
                 next_out			=> nnext,
                 start_out			=> start,
                 enable_out			=> enable,
                 stop_out			=> sstop,
                 State_out			=> state	
             );

    CLOCK_GEN: process is
    begin
        clk <= '0' after Tpw_clk, '1' after 2*Tpw_clk;
        wait for 2*Tpw_clk;
    end process CLOCK_GEN;

    --INHIBIT_GEN: process is
    --	begin
    --		inhibit_n <= '1' after 500000*Tpw_clk, '0' after 100000*Tpw_clk;
    --		wait for 500000*Tpw_clk;
    --	end process INHIBIT_GEN;

    TEST_PROCESS: process
    begin

        -- Provide the initial reset and initial inputs
        nreset<='0';
        wait for 100 us;
        -- release reset
        nreset <= '1';

        wait for 2 ms;
        inhibit_n <= '0';
        wait for 39 ms;
        vcap_inhib <= '1';
        wait for 1 ms;
        vcap_inhib <= '0';
        wait for 10 ms;
        inhibit_n <= '1';

        wait; -- will wait forever here
    end process;
END;

