Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;

ENTITY Test_Sequencer IS
END Test_Sequencer;

ARCHITECTURE behavior OF Test_Sequencer IS

	constant Tpw_clk	: time := 10.0 ns;
	
-- Component Declaration of the Module Under Test (MUT)
	component separation_buffer_v1
	port( 	
		clk						: in std_logic;
		nreset					: in std_logic;
		Inhibit_n				: in std_logic;
		Separation_Adj			: in std_logic_vector(3 downto 0); 
														
		Separation_out			: out std_logic_vector(7 downto 0);
		Cycle_Count_out			: out std_logic_vector(4 downto 0);
		State_out				: out std_logic_vector(3 downto 0)
		);
	end component;

-- inputs
	signal clk 			: std_logic := '0';
	signal nreset		: std_logic := '1';
	signal inhibit_n	: std_logic := '0';
	signal adj			: std_logic_vector(3 downto 0) := "0000";	
-- outputs
	signal val_out		: std_logic_vector(7 downto 0);
	signal cnt_out		: std_logic_vector(4 downto 0);
	signal state		: std_logic_vector(3 downto 0);
	

BEGIN
-- Instantiate the Module Under Test
MUT: separation_buffer_v1
	port map ( 	
			clk					=> clk,		
			nreset				=> nreset,			
			Inhibit_n			=> inhibit_n,
			Separation_Adj		=> adj,		
								
			Separation_out		=> val_out,	
			Cycle_Count_out		=> cnt_out,			        
			State_out		    => state	
			);

CLOCK_GEN: process is
	begin
		clk <= '0' after Tpw_clk, '1' after 2*Tpw_clk;
		wait for 2*Tpw_clk;
	end process CLOCK_GEN;

INHIBIT_GEN: process is
	begin
		inhibit_n <= '0' after 8*Tpw_clk, '1' after 16*Tpw_clk;
		wait for 16*Tpw_clk;
	end process INHIBIT_GEN;


TEST_PROCESS: process
	begin
	-- wait 100 ns for global reset to finish
	wait for 100 ns;
	-- Provide the initial reset and initial inputs
	nreset<='0'; adj<="0000";
	wait for 100 ns;
	-- release reset
	nreset <= '1';
	wait for 400 ns;
	

	wait; -- will wait forever here
	end process;
END;














