--- boodotbd-sequencer_timer_v2.vhd	2017-06-26 13:39:13.070260300 -0500
+++ sequencer_timer_v2.vhd	2017-07-14 08:37:30.515000200 -0500
@@ -28,7 +28,12 @@
 		Seq_Out_2			: out std_logic;
 		Seq_Out_3			: out std_logic;
 		Seq_Out_4			: out std_logic;
-		Seq_Out_5			: out std_logic
+		Seq_Out_5			: out std_logic;
+		next_out			: out std_logic;
+		start_out			: out std_logic;
+		enable_out			: out std_logic;
+		stop_out			: out std_logic;
+		State_out			: out std_logic_vector(3 downto 0)
 	);
 end sequencer_timer_v2;
 
@@ -36,51 +41,53 @@
 -- Sequence parameters
 
 	constant ENABLE_WIDTH	: std_logic_vector(15 downto 0) := X"0190"; -- 400, 0.125 ms ticks = 50 ms
+	--constant ENABLE_WIDTH	: std_logic_vector(15 downto 0) := X"0140"; -- 320, 0.125 ms ticks = 50 ms
 
 -- Syncronized Signals ----------------------------------------------
-	signal start				: std_logic;
-	signal stop_sequence		: std_logic;
-	signal inh_seq_d			: std_logic;
+	signal inhibit_n		: std_logic;
+	signal start			: std_logic;
+	signal stop_sequence	: std_logic;
+	signal inh_seq_d		: std_logic;
 	signal inhibit_seq		: std_logic;
-	signal vcap_inhb			: std_logic;
+	signal vcap_inhb		: std_logic;
 	
 -- The ms Timer -----------------------------------------------------
 	signal timer_ms			: std_logic_vector(15 downto 0);
 	signal start_timer		: std_logic;
 	signal rst_timer_ms		: std_logic;
-	signal tick_ms				: std_logic;
+	signal tick_ms			: std_logic;
     
 -- Sequence Pulse Separation ----------------------------------------
-	signal upd_timer			: std_logic_vector(7 downto 0);
-	signal ena_upd_timer		: std_logic;
-	signal next_pulse			: std_logic;
+	signal upd_timer		: std_logic_vector(7 downto 0);
+	signal ena_upd_timer	: std_logic;
+	signal next_pulse		: std_logic;
 	
 -- Sequence Pulse Width ---------------------------------------------
 	signal timer_1			: std_logic_vector(7 downto 0);
-	signal rst_timer_1	: std_logic;
+	signal rst_timer_1		: std_logic;
 	
 	signal timer_2			: std_logic_vector(7 downto 0);
-	signal rst_timer_2	: std_logic;
+	signal rst_timer_2		: std_logic;
 	
 	signal timer_3			: std_logic_vector(7 downto 0);
-	signal rst_timer_3	: std_logic;
+	signal rst_timer_3		: std_logic;
 	
 	signal timer_4			: std_logic_vector(7 downto 0);
-	signal rst_timer_4	: std_logic;
+	signal rst_timer_4		: std_logic;
 	
 	signal timer_5			: std_logic_vector(7 downto 0);
-	signal rst_timer_5	: std_logic;
+	signal rst_timer_5		: std_logic;
 	 
-	signal pulse_1		: std_logic; 
-	signal pulse_2		: std_logic; 
-	signal pulse_3		: std_logic; 
-	signal pulse_4		: std_logic; 
-	signal pulse_5		: std_logic; 
+	signal pulse_1			: std_logic; 
+	signal pulse_2			: std_logic; 
+	signal pulse_3			: std_logic; 
+	signal pulse_4			: std_logic; 
+	signal pulse_5			: std_logic; 
 
 -- Sequence Pulse Enable --------------------------------------------
-	signal enable				: std_logic;
-	signal enable_timer 		: std_logic_vector(15 downto 0);
-	signal rst_enable_timer : std_logic;
+	signal enable			: std_logic;
+	signal enable_timer		: std_logic_vector(15 downto 0);
+	signal rst_enable_timer	: std_logic;
 
 -- TIMER_STATE_MACHINE ----------------------------------------------
 	type STATE_TYPE is (idle, pulse_1_active, wait_for_next_1, pulse_2_active, wait_for_next_2, 
@@ -96,9 +103,16 @@
 -- The falling edge (fed) and leading edge (led) detector functions are 
 -- found in the included library file "pkg_081006.vhd".
 
-SYNC_START: led port map(sig_in=>Start_Sequence, 	clk=>clk, clrn=>nreset, pls_out=>start);
+inhibit_n <= not Start_Sequence;
+
+SYNC_START:	fed port map(sig_in=> inhibit_n, clk=>clk, clrn=>nreset, pls_out=>start);
 
 rst_enable_timer 	<= '1' when start = '1' else '0';
+--inhibit_seq 		<=	'1' when (vcap_inhb = '1' or stop_sequence = '1') else '0';
+
+start_out <= start;
+enable_out <= enable;
+stop_out <= stop_sequence;
 
 ------------------------------------------------------------------
 -- The 0.125 ms timer tick is generated by the CYCLE_TIMER and the
@@ -153,9 +167,9 @@
 end process UPDATE_TIMER;
 
 UPDATE_TEST:
-process(clk, nreset, upd_timer, PULSE_SEPARATION)
+process(clk, nreset, ena_upd_timer, upd_timer, PULSE_SEPARATION)
 begin
-	if   (nreset = '0')		then					next_pulse <= '0';
+	if   (nreset = '0' or ena_upd_timer = '0')		then		next_pulse <= '0';
 	elsif rising_edge(clk)	then
 		if (upd_timer = PULSE_SEPARATION) then	next_pulse <= '1';
 		else												next_pulse <= '0';
@@ -195,7 +209,7 @@
 
 ------------------------------------------------------------------
 SYNC_STOP:	fed port map(sig_in=>enable, 			clk=>clk, clrn=>nreset, pls_out=>stop_sequence);
-SYNC_VCAP:	led port map(sig_in=>Cap_Volt_Inh, 	clk=>clk, clrn=>nreset, pls_out=>vcap_inhb);
+SYNC_VCAP:	led port map(sig_in=>Cap_Volt_Inh, 		clk=>clk, clrn=>nreset, pls_out=>vcap_inhb);
 
 SYNC_INHIBIT: process(clk, nreset, vcap_inhb, stop_sequence)
 begin
@@ -361,6 +375,7 @@
 	rst_timer_5				<= '0';
 	
 	ena_upd_timer			<= '0';
+	State_out				<= X"0";
 	
 	------------------------------------------------------------------
 	-- This state machine does only the timing for the separation between pulses.
@@ -370,6 +385,7 @@
 
 	case CS is
 		when idle =>
+			State_out <= X"0";
 			start_timer <= '1';				-- this holds the timer in reset actually
 			if (start = '1') then						NS <= pulse_1_active;
 			else												NS <= idle; -- wait
@@ -377,10 +393,12 @@
 			
 	------------------------------------------------------------------
 		when pulse_1_active =>							NS <= wait_for_next_1;
+			State_out <= X"1";
 			ena_upd_timer 		<= '0';		-- reset timing until next pulse
 			rst_timer_1 		<= '1';		-- starting pulse 1
 			
 		when wait_for_next_1 =>
+			State_out <= X"2";
 			ena_upd_timer 		<= '1';		-- timing until next pulse
 		
 			if (next_pulse = '1') then					NS <= pulse_2_active;
@@ -388,10 +406,12 @@
 			end if;
 			
 		when pulse_2_active =>							NS <= wait_for_next_2;
-			ena_upd_timer		<= '0';		-- reset timing until next pulse
+			State_out <= X"3";
+			ena_upd_timer 		<= '0';		-- reset timing until next pulse
 			rst_timer_2			<= '1';		-- starting pulse 2
 			
 		when wait_for_next_2 =>
+			State_out <= X"4";
 			ena_upd_timer		<= '1';		-- timing until next pulse
 		
 			if (next_pulse = '1') then					NS <= pulse_3_active;
@@ -399,10 +419,12 @@
 			end if;
 			
 		when pulse_3_active =>							NS <= wait_for_next_3;
-			ena_upd_timer		<= '0';		-- reset timing until next pulse
+			State_out <= X"5";
+			ena_upd_timer 		<= '0';		-- reset timing until next pulse
 			rst_timer_3			<= '1';		-- starting pulse 3
 			
 		when wait_for_next_3 =>
+			State_out <= X"6";
 			ena_upd_timer		<= '1';		-- timing until next pulse
 		
 			if (next_pulse = '1') then					NS <= pulse_4_active;
@@ -410,10 +432,12 @@
 			end if;
 			
 		when pulse_4_active =>							NS <= wait_for_next_4;
-			ena_upd_timer		<= '0';		-- reset timing until next pulse
+			State_out <= X"7";
+			ena_upd_timer 		<= '0';		-- reset timing until next pulse
 			rst_timer_4			<= '1';		-- starting pulse 4
 			
 		when wait_for_next_4 =>
+			State_out <= X"8";
 			ena_upd_timer		<= '1';		-- timing until next pulse
 		
 			if (next_pulse = '1') then					NS <= pulse_5_active;
@@ -421,10 +445,12 @@
 			end if;
 			
 		when pulse_5_active =>							NS <= wait_for_next_5;
-			ena_upd_timer		<= '0';		-- reset timing until next pulse
+			State_out <= X"9";
+			ena_upd_timer 		<= '0';		-- reset timing until next pulse
 			rst_timer_5			<= '1';		-- starting pulse 5
 			
 		when wait_for_next_5 =>
+			State_out <= X"A";
 			ena_upd_timer		<= '1';		-- timing until next pulse
 		
 			if (next_pulse = '1') then					NS <= pulse_1_active;	
@@ -432,17 +458,21 @@
 			end if;
 			
 		when done =>
+			State_out <= X"B";
 			NS <= idle;
 			
 		when others =>
+			State_out <= X"C";
 			NS <= idle;
 	end case;
 end process SM_NEXT_STATE;
 
-Seq_Out_1 <= pulse_1;
-Seq_Out_2 <= pulse_2;
-Seq_Out_3 <= pulse_3;
-Seq_Out_4 <= pulse_4;
-Seq_Out_5 <= pulse_5;
+Seq_Out_1 <= pulse_1 when enable = '1' else '0';
+Seq_Out_2 <= pulse_2 when enable = '1' else '0';
+Seq_Out_3 <= pulse_3 when enable = '1' else '0';
+Seq_Out_4 <= pulse_4 when enable = '1' else '0';
+Seq_Out_5 <= pulse_5 when enable = '1' else '0';
+
+next_out <= next_pulse;
 
 end main;
\ No newline at end of file
