-- Package of my types and components ------------------------------
-- Date initiated: April 3, 2008
-- Author: Craig Drennan
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

------------------------------------------------------------------
-- Types, Constants, Components, etc.
package pkg_081006 is
	constant WID4 : positive := 4;
	constant WID6 : positive := 6;
	constant WID8 : positive := 8;
	constant WID9 : positive := 9;
	constant WID14: positive := 14;
	constant WID16: positive := 16;
	constant WID24: positive := 24;
	constant WID32: positive := 32;
	
	component dffr is
		generic(wid: positive);
		port(signal d    : IN STD_LOGIC_VECTOR(wid-1 downto 0); 
			 signal clk  : IN STD_LOGIC; 
			 signal clrn : IN STD_LOGIC; 
			 signal prn  : IN STD_LOGIC; 
			 signal ena  : IN STD_LOGIC; 
			 signal q    : OUT STD_LOGIC_VECTOR(wid-1 downto 0) );
	end component;

	component delay is
		generic(len: positive);
		port(signal d    : IN STD_LOGIC; 
			 signal clk  : IN STD_LOGIC; 
			 signal clrn : IN STD_LOGIC; 
			 signal prn  : IN STD_LOGIC; 
			 signal ena  : IN STD_LOGIC; 
			 signal q    : OUT STD_LOGIC_VECTOR(len-1 downto 0) );
	end component;

	component vdffe is
		port(signal d    : IN STD_LOGIC; 
			 signal clk  : IN STD_LOGIC; 
			 signal clrn : IN STD_LOGIC; 
			 signal prn  : IN STD_LOGIC; 
			 signal ena  : IN STD_LOGIC; 
			 signal q    : OUT STD_LOGIC );
	end component;

	component counter is
		generic(nbits: positive);
		port( clk, clrn, ena, load, updn : in  std_logic;
			  ld_data  : in  std_logic_vector(nbits-1 downto 0);
			  flag_cnt : in  std_logic_vector(nbits-1 downto 0);
			  flag_out : out std_logic;
			  cnt_out  : out std_logic_vector(nbits-1 downto 0) );
	end component;

	component countby_n is
		generic(nbits: positive; nstep: positive := 4);
		port( clk, clrn, ena, load	: in  std_logic;
			  ld_data  : in  std_logic_vector(nbits-1 downto 0);
			  step	   : in  std_logic_vector(nstep-1 downto 0);
			  flag_cnt : in  std_logic_vector(nbits-1 downto 0);
			  flag_out : out std_logic;
			  cnt_out  : out std_logic_vector(nbits-1 downto 0) );
	end component;

	component countby_n_VER2 is
		generic(nbits: positive; nstep: positive := 4);
		port( clk, clrn, ena, load	: in  std_logic;
			  ld_data  : in  std_logic_vector(nbits-1 downto 0);
			  step	   : in  std_logic_vector(nstep-1 downto 0);
			  flag_cnt : in  std_logic_vector(nbits-1 downto 0);
			  flag_out : out std_logic;
			  cnt_out  : out std_logic_vector(nbits-1 downto 0) );
	end component;

	component countby2 is
		generic(nbits: positive);
		port( clk, clrn, ena, load, updn : in  std_logic;
			  ld_data  : in  std_logic_vector(nbits-1 downto 0);
			  flag_cnt : in  std_logic_vector(nbits-1 downto 0);
			  flag_out : out std_logic;
			  cnt_out  : out std_logic_vector(nbits-1 downto 0) );
	end component;

	component count_updn_by_n is
		generic(nbits: positive; nstep: positive := 4);
		port( clk, clrn, ena, load, up_dn_n	: in  std_logic;
			  ld_data  		: in  std_logic_vector(nbits-1 downto 0);
			  step	   		: in  std_logic_vector(nstep-1 downto 0);
			  flag_cnt_up 	: in  std_logic_vector(nbits-1 downto 0);
			  flag_out_up 	: out std_logic;
			  flag_cnt_dn 	: in  std_logic_vector(nbits-1 downto 0);
			  flag_out_dn 	: out std_logic;
			  cnt_out  		: out std_logic_vector(nbits-1 downto 0) );
	end component;

	component led is 
		port(signal sig_in  : IN STD_LOGIC; 
			 signal clk     : IN STD_LOGIC; 
			 signal clrn    : IN STD_LOGIC; 
			 signal pls_out : OUT STD_LOGIC );
	end component;

	component fed is 
		port(signal sig_in  : IN STD_LOGIC; 
			 signal clk     : IN STD_LOGIC; 
			 signal clrn    : IN STD_LOGIC; 
			 signal pls_out : OUT STD_LOGIC );
	end component;

	component sync is 
		port(signal sig_in  : IN STD_LOGIC; 
			 signal clk     : IN STD_LOGIC; 
			 signal clrn    : IN STD_LOGIC; 
			 signal sig_out : OUT STD_LOGIC );
	end component;

	component serial_clk is
	   port(signal clk        : in std_logic;
			signal clrn       : in std_logic;
			signal sclk_ena   : in std_logic; 
			signal sr_clk     : out std_logic;
			signal sclk       : out std_logic );
	end component;
	
	component step_up_ndn is
	generic(nbits: positive; nstep: positive := 4);
	   port(  clk      		: in  std_logic;
			  clrn     		: in  std_logic;
			  ena			: in  std_logic;
			  up_ndn		: in  std_logic;
			  value_in 		: in  std_logic_vector(nbits-1 downto 0);
			  step	   		: in  std_logic_vector(nstep-1 downto 0);
			  flag_cnt_up	: in  std_logic_vector(nbits-1 downto 0);
			  flag_cnt_dn	: in  std_logic_vector(nbits-1 downto 0);
			  flag_out_up	: out std_logic;
			  flag_out_dn	: out std_logic;
			  value_out  	: out std_logic_vector(nbits-1 downto 0) );
	end component;

end pkg_081006;

------------------------------------------------------------------
-- Entities
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

-- Variable Width DFF Register ------------------------------
-- Date initiated: April 3, 2008
-- Author: Craig Drennan
entity dffr is 
	generic(wid: positive);
	port(signal d    : IN STD_LOGIC_VECTOR(wid-1 downto 0); 
		 signal clk  : IN STD_LOGIC; 
		 signal clrn : IN STD_LOGIC; 
		 signal prn  : IN STD_LOGIC; 
		 signal ena  : IN STD_LOGIC; 
		 signal q    : OUT STD_LOGIC_VECTOR(wid-1 downto 0) );
end dffr;

architecture behavior of dffr is
begin
	process(clk, clrn, prn, ena)
	begin
		if(clrn = '0')  then q <= (others=>'0');
		elsif(prn = '0')then q <= (others=>'1');
		elsif(clk'event and clk='1' and ena='1')
		then q <= d;
		end if;
	end process;
end behavior;
---------------------------------------------------------------

------------------------------------------------------------------
-- Entities
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

-- Variable Length Delay (single width) --------------------------
-- Date initiated: October 10, 2008
-- Author: Craig Drennan
entity delay is 
	generic(len: positive);
	port(signal d    : IN STD_LOGIC; 
		 signal clk  : IN STD_LOGIC; 
		 signal clrn : IN STD_LOGIC; 
		 signal prn  : IN STD_LOGIC; 
		 signal ena  : IN STD_LOGIC; 
		 signal q    : OUT STD_LOGIC_VECTOR(len-1 downto 0) );
end delay;

architecture behavior of delay is
begin
	process(clk, clrn, prn, ena)
		variable qreg: std_logic_vector(len-1 downto 0);
	begin
		if(clrn = '0')  then qreg := (others=>'0');
		elsif(prn = '0')then qreg := (others=>'1');
		elsif(rising_edge(clk) and ena='1')then
			for i in 0 to len-2 loop
				qreg(i) := qreg(i+1);
			end loop;
			qreg(len-1) := d;
		end if;
		q <= qreg;
	end process;
end behavior;
---------------------------------------------------------------

------------------------------------------------------------------
-- Entities
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

-- Variable Width DFF Register ------------------------------
-- Date initiated: October 8, 2008
-- Author: Craig Drennan
entity vdffe is 
	port(signal d    : IN STD_LOGIC; 
		 signal clk  : IN STD_LOGIC; 
		 signal clrn : IN STD_LOGIC; 
		 signal prn  : IN STD_LOGIC; 
		 signal ena  : IN STD_LOGIC; 
		 signal q    : OUT STD_LOGIC );
end vdffe;

architecture behavior of vdffe is
begin
	process(clk, clrn, prn, ena)
		variable qreg: std_logic;
	begin
		if(clrn = '0')  then qreg := '0';
		elsif(prn = '0')then qreg := '1';
		elsif(clk'event and clk='1' and ena='1')
						then qreg := d;
		end if;
		q <= qreg;
	end process;
end behavior;
---------------------------------------------------------------

-- Variable Size Counter --------------------------------------
-- Date initiated: April 3, 2008
-- Author: Craig Drennan
--
-- This counter is up/down, async reset, sync load and will 
-- set a flag at a desired flag count.
--
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity counter is
	generic(nbits: positive);
	port( clk, clrn, ena, load, updn : in  std_logic;
		  ld_data  : in  std_logic_vector(nbits-1 downto 0);
		  flag_cnt : in  std_logic_vector(nbits-1 downto 0);
		  flag_out : out std_logic;
		  cnt_out  : out std_logic_vector(nbits-1 downto 0) );
end counter;

architecture behavior of counter is
	signal count : std_logic_vector(nbits-1 downto 0);
	signal maxcnt: std_logic_vector(nbits-1 downto 0);
	signal mincnt: std_logic_vector(nbits-1 downto 0);
begin
	process(clk, clrn, load, ena, count, flag_cnt)
	begin
		maxcnt <= (others => '1'); -- sets all bits high
		mincnt <= (others => '0'); -- sets all bits low
		
		if (clrn = '0')  then count <= (others => '0');		
		elsif(clk'event and clk='1'and load='1')then count <= ld_data;
		elsif(clk'event and clk='1'and ena='1') then
		  if   (count/=maxcnt and updn='1') then count <= count + 1;
		  elsif(count/=mincnt and updn='0') then count <= count - 1;
		  else                                   count <= count;
		  end if;
		else                                     count <= count;
		end if;	
		
		cnt_out <= count;
		
		if(count = flag_cnt) then flag_out <= '1';
		else					  flag_out <= '0';
		end if;
		
	end process;
end behavior;	

-- Variable Step Size Counter -----------------------------------
-- Date initiated: May 17, 2010
-- Author: Craig Drennan
--
-- This counter has a settable step increment, async reset, sync load 
-- and will set a flag at a desired flag count.
--
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity countby_n is
	generic(nbits: positive; nstep: positive := 4);
	port( clk, clrn, ena, load	: in  std_logic;
		  ld_data  : in  std_logic_vector(nbits-1 downto 0);
		  step	   : in  std_logic_vector(nstep-1 downto 0);
		  flag_cnt : in  std_logic_vector(nbits-1 downto 0);
		  flag_out : out std_logic;
		  cnt_out  : out std_logic_vector(nbits-1 downto 0) );
end countby_n;

architecture behavior of countby_n is
	signal count	: std_logic_vector(nbits-1 downto 0);
	signal end_count: std_logic_vector(nbits-1 downto 0);

begin
	process(clk, clrn, load, ena, count, end_count, flag_cnt)
	begin		
		if (clrn = '0')  then 
			count 		<= (others => '0');	
			end_count	<= (others => '0');		
		elsif(clk'event and clk='1'and load='1')then 
			count 		<= ld_data;
			end_count	<= flag_cnt;
			
		elsif(clk'event and clk='1'and ena='1') then count <= count + step;
		else                                         count <= count;
		end if;	
		
		cnt_out <= count;
		
		if(count > end_count) then flag_out <= '1';
		else					   flag_out <= '0';
		end if;
		
	end process;
end behavior;

-- Variable Step Size Counter  VERSION 2-----------------------------

-- THIS VERSION USES <= TEST FOR END COUNT FLAG INSTEAD OF < ONLY.
-- Date initiated: March 1, 2011
-- Author: Craig Drennan
--
-- This counter has a settable step increment, async reset, sync load 
-- and will set a flag at a desired flag count.
--
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity countby_n_VER2 is
	generic(nbits: positive; nstep: positive := 4);
	port( clk, clrn, ena, load	: in  std_logic;
		  ld_data  : in  std_logic_vector(nbits-1 downto 0);
		  step	   : in  std_logic_vector(nstep-1 downto 0);
		  flag_cnt : in  std_logic_vector(nbits-1 downto 0);
		  flag_out : out std_logic;
		  cnt_out  : out std_logic_vector(nbits-1 downto 0) );
end countby_n_VER2;

architecture behavior of countby_n_VER2 is
	signal count	: std_logic_vector(nbits-1 downto 0);
	signal end_count: std_logic_vector(nbits-1 downto 0);

begin
	process(clk, clrn, load, ena, count, end_count, flag_cnt)
	begin		
		if (clrn = '0')  then 
			count 		<= (others => '0');	
			end_count	<= (others => '0');		
		elsif(clk'event and clk='1'and load='1')then 
			count 		<= ld_data;
			end_count	<= flag_cnt;
			
		elsif(clk'event and clk='1'and ena='1') then count <= count + step;
		else                                         count <= count;
		end if;	
		
		cnt_out <= count;
		
		if(count >= end_count) then flag_out <= '1';
		else					   flag_out <= '0';
		end if;
		
	end process;
end behavior;

---------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

-- Syncronizing Leading Edge Detector ------------------------------
-- Date initiated: April 4, 2008
-- Author: Craig Drennan
entity led is 
	port(signal sig_in  : IN STD_LOGIC; 
		 signal clk  : IN STD_LOGIC; 
		 signal clrn : IN STD_LOGIC; 
		 signal pls_out : OUT STD_LOGIC );
end led;

architecture behavior of led is
	signal qreg1: std_logic;
	signal qreg2: std_logic;
begin	
	process(clk, clrn, qreg1, qreg2)
	begin
		if(clrn = '0')  then 
			qreg1 <= '0';
			qreg2 <= '0';
		elsif rising_edge(clk) then
			qreg1 <= sig_in;
			qreg2 <= qreg1;
		end if;		
		
		pls_out <= qreg1 and (not qreg2);
	end process;
end behavior;
---------------------------------------------------------------		

---------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

-- Syncronizing Falling Edge Detector ------------------------------
-- Date initiated: Feb 17, 2017
-- Author: Brian Schupbach
entity fed is 
	port(signal sig_in  : IN STD_LOGIC; 
		 signal clk  : IN STD_LOGIC; 
		 signal clrn : IN STD_LOGIC; 
		 signal pls_out : OUT STD_LOGIC );
end fed;

architecture behavior of fed is
	signal qreg1: std_logic;
	signal qreg2: std_logic;
begin	
	process(clk, clrn, qreg1, qreg2)
	begin
		if(clrn = '0')  then 
			qreg1 <= '0';
			qreg2 <= '0';
		elsif rising_edge(clk) then
			qreg1 <= sig_in;
			qreg2 <= qreg1;
		end if;		
		
		pls_out <= (not qreg1) and qreg2;
	end process;
end behavior;
--------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

-- Two Register Syncronizer ------------------------------
-- Date initiated: April 4, 2008
-- Author: Craig Drennan
entity sync is 
	port(signal sig_in  : IN STD_LOGIC; 
		 signal clk     : IN STD_LOGIC; 
		 signal clrn    : IN STD_LOGIC; 
		 signal sig_out : OUT STD_LOGIC );
end sync;

architecture behavior of sync is
	signal qreg1: std_logic;
	signal qreg2: std_logic;
begin	
	process(clk, clrn, qreg1, qreg2)
	begin
		if(clrn = '0')  then 
			qreg1 <= '0';
			qreg2 <= '0';
		elsif rising_edge(clk) then
			qreg1 <= sig_in;
			qreg2 <= qreg1;
		end if;
		
		sig_out <= qreg2;
	end process;
end behavior;
---------------------------------------------------------------		
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

-- Serial Clock Generator (2 clocks high, 2 clocks low) with shift reg enable--
-- Date initiated: October 6, 2008
-- Author: Craig Drennan		

----------------------------------------------------------------------
-- This is the serial clock that clocks data in and out of the flash
-- memory chip.  There are actually two clock type signals out of
-- this component.  There is the serial clock (sclk) with a period of 
-- 4 systems clocks and a 50% duty cycle, and there is a single system
-- clock wide shift register clock enable pulse that occurs once every
-- 4 system clocks.
--            _   _   _   _   _   _   _   _   _   _   _   _   _   _   _                                            
-- sys_clk  _| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |  
--              ____________________________________
-- fm_sclk_ena_|                                    |____________________
--      	       _______         _______         _______          
-- fm_sclk1 ______|       |_______|       |_______|       |______________
--                      __              __              __                  
-- sr_clk_ena _________|  |____________|  |____________|  |______________	

entity serial_clk is
   port(clk        : in std_logic;
		clrn       : in std_logic;
		sclk_ena   : in std_logic; 
		sr_clk     : out std_logic;
		sclk       : out std_logic
		);
end serial_clk;

architecture main of serial_clk is
-- FM SCLK and Shift Register Clock Generating State Machine declarations ---------
	type CLK_STATE_TYPE is (low1, low2, high1, high2);
	signal CCS, CNS  : CLK_STATE_TYPE;

begin
-- FM SCLK State Machine Process definition ---------------------------
	sclk_sm_reg: process (clk, clrn)	
	begin
		if(clrn='0')                then CCS <= low1;
		elsif(clk'event and clk='1')then CCS <= CNS;
		end if;
	end process; -- sclk_sm_reg
	
	sclk_sm_next_state: process (CCS, sclk_ena)
	begin
		case CCS is
			when low1   =>
				if(sclk_ena ='1') then CNS <= low2; else CNS <= low1;
				end if; 
			when low2   => 
				if(sclk_ena ='1') then CNS <= high1; else CNS <= low1;
				end if; 
			when high1  =>
				if(sclk_ena ='1') then CNS <= high2; else CNS <= low1;
				end if; 
			when high2  => CNS <= low1  ;
		end case; 		
	end process; -- sclk_sm_combinatorial
	
	sclk_sm_outputs: process (clk, CCS)
	begin
		if rising_edge(clk) then
		case CNS is
			when low1   => sr_clk   <= '0'; sclk     <= '0';  
			when low2   => sr_clk   <= '0'; sclk     <= '0'; 
			when high1  => sr_clk   <= '0'; sclk     <= '1'; 
			when high2  => sr_clk   <= '1'; sclk     <= '1'; 
		end case; 
		end if;		
	end process; -- sclk_sm_outputs
end main;
	
-----------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

-- Step input value up or down by variable amount and test endpoints --
-- Date initiated: February 25, 2011
-- Author: Craig Drennan
entity step_up_ndn is 
	generic(nbits: positive; nstep: positive := 4);
	port(clk      		: in  std_logic;
		 clrn     		: in  std_logic;
		 ena			: in  std_logic;
		 up_ndn			: in  std_logic;
		 value_in 		: in  std_logic_vector(nbits-1 downto 0);
		 step	   		: in  std_logic_vector(nstep-1 downto 0);
		 flag_cnt_up	: in  std_logic_vector(nbits-1 downto 0);
		 flag_cnt_dn	: in  std_logic_vector(nbits-1 downto 0);
		 flag_out_up	: out std_logic;
		 flag_out_dn	: out std_logic;
		 value_out  	: out std_logic_vector(nbits-1 downto 0) );
end step_up_ndn;

architecture behavior of step_up_ndn is
	signal value1	: std_logic_vector(nbits-1 downto 0);

begin	
	process(clk, clrn, up_ndn, value1, value_in, step,
			flag_cnt_up, flag_cnt_dn)
	begin
		if   (clrn = '0') then
			value_out 	<= (others => '0');
			flag_out_up <= '0';
			flag_out_dn <= '0';

		elsif rising_edge(clk) then
			if (ena = '1') then
				if   (up_ndn = '1') then -- stepping up --
					value1 <= value_in + step;
					if   (value1 =	flag_cnt_up) then
						flag_out_up 	<= '1';
						value_out 		<= value1;
					elsif(value1 >  flag_cnt_up) then
						flag_out_up 	<= '1';
						value_out 		<= flag_cnt_up;
					elsif(value1 < value_in) then --roll over--
						flag_out_up 	<= '1';
						value_out 		<= flag_cnt_up;
					else
						flag_out_up 	<= '0';
						value_out 		<= value1;
					end if;
						
				else	-- stepping down --				 
					value1 <= value_in - step;
					if   (value1 =	flag_cnt_dn) then
						flag_out_dn 	<= '1';
						value_out 		<= value1;
					elsif(value1 <  flag_cnt_dn) then
						flag_out_dn 	<= '1';
						value_out 		<= flag_cnt_dn;
					elsif(value1 > value_in) then --roll under--
						flag_out_dn 	<= '1';
						value_out 		<= flag_cnt_dn;
					else
						flag_out_dn 	<= '0';
						value_out 		<= value1;
					end if;				
				end if;	
			end if;
		end if;	
	end process;
end behavior;		